#!/usr/bin/env python
# encoding: utf-8

"""
Copyright (C) 2011  Julien Le Sech - www.idreammicro.com

This file is part of the MAX7219 library for Bus Pirate.

This library is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option)b any later
version.

This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for moredetails.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/
"""

from serial.serialutil import SerialException

from pyBusPirateLite.SPI import *

from max7219_constants import *

class ShutdownMode:
    """
    Shutdown modes.
    """
    Shutdown		= 0x00
    NormalOperation	= 0x01

class DisplayTest:
    """
    Display test modes.
    """
    NormalOperation	= 0x00
    TestMode		= 0x01

class ScanLimit:
    """
    Scan limits.
    """
    Digit0		= 0x00
    Digit0To1	= 0x01
    Digit0To2	= 0x02
    Digit0To3	= 0x03
    Digit0To4	= 0x04
    Digit0To5	= 0x05
    Digit0To6	= 0x06
    Digit0To7	= 0x07

class Intensity:
    """
    Intensities.
    """
    Level0	= 0x00
    Level1	= 0x01
    Level2	= 0x02
    Level3	= 0x03
    Level4	= 0x04
    Level5	= 0x05
    Level6	= 0x06
    Level7	= 0x07
    Level8	= 0x08
    Level9	= 0x09
    Level10	= 0x0A
    Level11	= 0x0B
    Level12	= 0x0C
    Level13	= 0x0D
    Level14	= 0x0E
    Level15 = 0x0F

class DecodeMode:
    """
    Decode modes.
    """
    NoDecode	= 0x00
    Digit0		= 0x01
    Digit1      = 0x02
    Digit2      = 0x04
    Digit3      = 0x08
    Digit4      = 0x10       
    Digit5      = 0x20
    Digit6      = 0x40
    Digit7      = 0x80
    AllDigits   = 0xFF

class Digit:
    """
    Digits.
    """
    Digit0	= 0x01
    Digit1	= 0x02
    Digit2	= 0x03
    Digit3	= 0x04
    Digit4	= 0x05
    Digit5	= 0x06
    Digit6	= 0x07
    Digit7	= 0x08

class Character:
    """
    Characters.
    """
    Zero	= 0x00
    One		= 0x01
    Two		= 0x02
    Three	= 0x03
    Four	= 0x04
    Five	= 0x05
    Six		= 0x06
    Seven	= 0x07
    Eight	= 0x08
    Nine	= 0x09
    Dash	= 0x0A
    E		= 0x0B
    H		= 0x0C
    L		= 0x0D
    P		= 0x0E
    Blank	= 0x0F

class Segment:
    """
    Segments.
    """
    DP	= 0x80
    A	= 0x40
    B	= 0x20
    C	= 0x10
    D	= 0x08
    E	= 0x04
    F	= 0x02
    G	= 0x01

class Max7219:
    """
    This class allow to drive a MAX7219 using Bus Pirate.
    """
    
    def __init__(self, port, baudrate):
        self.port = port
        self.baudrate = baudrate
        self.spi = SPI(self.port, self.baudrate)

    def initialize(self):
        """
        Initialize and configure Bus Pirate to drive a MAX7219.
        """
        # Enter BitBang mode.
        self.spi.BBmode()
        # Enter SPI mode.
        self.spi.enter_SPI()
        # Set SPI speed.
        self.spi.set_speed(SPISpeed._1MHZ)
        # Configure pins.
        self.spi.cfg_pins(PinCfg.POWER | PinCfg.CS)
        # Configure SPI.
        self.spi.cfg_spi(SPICfg.OUT_TYPE | SPICfg.CLK_EDGE)
    
    def set_shutdown_mode(self, shutdown_mode):
        """
        Set shutdown mode.
        """
        self.write(REG_SHUTDOWN, shutdown_mode)
    
    def set_display_test(self, display_test):
        """
        Set display test mode.
        """
        self.write(REG_DISPLAY_TEST, display_test)

    def set_scan_limit(self, scan_limit):
        """
        Set scan limit.
        """
        self.write(REG_SCAN_LIMIT, scan_limit)
        
    def set_intensity(self, intensity):
        """
        Set intensity.
        """
        self.write(REG_INTENSITY, intensity)
        
    def set_decode_mode(self, decode_mode):
        """
        Set decode mode.
        """
        self.write(REG_DECODE_MODE, decode_mode)
    
    def write_digit(self, digit, value, decimal_point = False):
        """
        Write digit value.
        """
        if decimal_point == True:
            value = value | 0x80
        self.write(digit, value)
        
    def write(self, address, value):
        """
        Write register value.
        """
        self.spi.CS_Low()
        self.spi.bulk_trans(2, [address, value])
        self.spi.CS_High()
